function str = jstruct2mlstruct(jstr,varargin)
%JSTRUCT2MLSTRUCT Java structure object to Matlab structure object.
%   Mobj = JSTRUCT2MLSTRUCTC(Jobj) returns a Matlab structure object
%   Mobj converted from the corresponding Java object.
%
%   Mobj = JSTRUCT2MLSTRUCTC(Jobj,Eigv) also provides the imposed 
%   eigenvalues on a structure. Eigv is of type 
%   se.umu.stratigraph.core.util.Eigenvalue.

import se.umu.stratigraph.core.structure.*;
import se.umu.stratigraph.core.util.Eigenvalue;

% If the structure is empty do a quick return
if jstr.isEmpty()
	str = [];
  return
end

if nargin > 1 && ~isempty(varargin{1})
	eigvo = varargin{1};
else
	eigvo = jstr.getRegularIndices().getEigenvalues();
end
rilo = jstr.getRegularIndices().toArray();
leneig = length(eigvo);
eigv = zeros(leneig,1);
J = cell(leneig,1);

cname = strsplit(class(jstr),'.');
switch cname{end}
  case 'PencilStructure'
    R = double(jstr.getIndices(BlockType.R).getSegreBlocks());
    L = double(jstr.getIndices(BlockType.L).getSegreBlocks());
    %N = double(jstr.getIndices(BlockType.N).getSegreBlocks());
    I = [];
    k = 0;
    for ii = 1:leneig
      blk = double(rilo(ii).getSegreBlocks());
      if isempty(eigvo(ii))
        k = k+1;
        eigv(k) = NaN;
        J{k}    = blk;
      elseif eigvo(ii).isInfinite()
        I       = blk;
      elseif eigvo(ii).isZero()
        k = k+1;
        eigv(k) = 0.0;
        J{k}    = blk;
      else
        k = k+1;
        eigv(k) = eigvo(ii).real() + 1i*eigvo(ii).imag();
        J{k}    = blk;
      end
    end
    eigv = eigv(1:k);
    J = J(1:k);
    str = pstruct(R,L,J,eigv,I);
  case 'MatrixStructure'
    for k = 1:leneig
      blk = double(rilo(k).getSegreBlocks());
      if isempty(eigvo(k))
        eigv(k) = NaN;
        J{k}    = blk;
      elseif eigvo(k).isZero()
        eigv(k) = 0.0;
        J{k}    = blk;
      else
        eigv(k) = eigvo(k).real() + 1i*eigvo(k).imag();
        J{k}    = blk;
      end
    end
    str = mstruct(J,eigv);
  case 'MatrixPairStructure'
    % One of R and L will be empty
    Ro = jstr.getIndices(BlockType.R);
    if ~isempty(Ro)
      R = double(Ro.getSegreBlocks());
      L = [];
    else
      R = [];
      L = double(jstr.getIndices(BlockType.L).getSegreBlocks());
    end
    
    for k = 1:leneig
      blk = double(rilo(k).getSegreBlocks());
      if isempty(eigvo(k))
        eigv(k) = NaN;
        J{k}    = blk;
      elseif eigvo(k).isZero()
        eigv(k) = 0.0;
        J{k}    = blk;
      else
        eigv(k) = eigvo(k).real() + 1i*eigvo(k).imag();
        J{k}    = blk;
      end
    end
    str = ssstruct(R,L,J,eigv);
  case 'StateSpaceStructure'
    R = double(jstr.getIndices(BlockType.R).getSegreBlocks());
    L = double(jstr.getIndices(BlockType.L).getSegreBlocks());
    N = double(jstr.getIndices(BlockType.N).getSegreBlocks());
    for k = 1:leneig
      blk = double(rilo(k).getSegreBlocks());
      if isempty(eigvo(k))
        eigv(k) = NaN;
        J{k}    = blk;
      elseif eigvo(k).isZero()
        eigv(k) = 0.0;
        J{k}    = blk;
      else
        eigv(k) = eigvo(k).real() + 1i*eigvo(k).imag();
        J{k}    = blk;
      end
    end
    str = ssstruct(R,L,J,eigv,N);
  case 'PolyMatrixStructure'
    error('Polynomial matrices are not implemented.')
  otherwise
    error('Not a recognized structure: %s',cname{end});
end
	