function [sgc, winID, label] = sgwindata(label,sgc)
%SGWINDATA Get or set data associated with a StratiGraph window.
%   [SGCom, winID, Label] = SGWINDATA(Label) checks if any StratiGraph
%   window exists with the label Label. If so, the associated sgcom object
%   and StratiGraph winID is returned. If input Label is an empty string or
%   zero, then SGCom, winID, and Label for the first found opened
%   StratiGraph window is returned. If the StratiGraph window has not sgcom
%   object associated to it, an empty sgcom object is created and returned
%   in SGCom.
%
%   [SGCom, winID, Label] = SGWINDATA(Label, SGCom) associate the sgcom 
%   object SGCom with the StratiGraph window with label Label. If Label is 
%   an empty string or zero, SGCom is associated with the first found 
%   opened StratiGraph window.
%
%   [...] = SGWINDATA(winID) or 
%   [...] = SGWINDATA(winID,SGCom) same as above but where the StratiGraph 
%   window is identified with its winID object.
%
%   The function throws an appropriate error when no StratiGraph window is 
%   found.
%
%   See also SGLABELS.

global MCS_PREF

import se.umu.stratigraph.core.util.WindowID;

if ~isfield(MCS_PREF,'sgwin') || isempty(MCS_PREF.sgwin) || isempty(MCS_PREF.sgwin(1).winID)
	error(['No instance of StratiGraph is started. Start StratiGraph' ...
	' with ''sgopen''.']);
end

if isempty(label) || (isscalar(label) && label == 0)
	winID = MCS_PREF.sgwin(1).winID;
	% Find the first SG window open
	while ~WindowID.exist(winID)
		% The window has probably been closed, remove from list
		MCS_PREF.sgwin(1) = [];
			
		% Check if no windows are left
		if isempty(MCS_PREF.sgwin)
			error(['Probably no instance of StratiGraph is started. Start StratiGraph' ...
			' with ''sgopen''.']);
		end
		winID = MCS_PREF.sgwin(1).winID;
	end
	label = MCS_PREF.sgwin(1).label;
	if nargin == 1
    if isfield(MCS_PREF.sgwin(1),'com')
      sgc = MCS_PREF.sgwin(1).com;
    else
      % Create a new sgcom object and associate it to the window
      sgc = sgcom();
      MCS_PREF.sgwin(1).com = sgc;
    end
	else
		MCS_PREF.sgwin(1).com = sgc;
	end			
elseif isa(label,'WindowID')
	% Find StratiGraph Window
	[b,nsg] = ismemberwin(label, MCS_PREF.sgwin(:).winID);
	if b && WindowID.exist(label)
		label = MCS_PREF.sgwin(nsg).label;
		if nargin == 1
      if isfield(MCS_PREF.sgwin(nsg),'com')
        sgc = MCS_PREF.sgwin(nsg).com;
      else
        % Create a new sgcom object and associate it to the window
        sgc = sgcom();
        MCS_PREF.sgwin(nsg).com = sgc;
      end
		else
			MCS_PREF.sgwin(nsg).com = sgc;
		end			
	else
		if b
			% The window has probably been closed, remove from list
			MCS_PREF.sgwin(nsg) = [];
		end
		error('The StratiGraph window is not found (may have been closed).')
	end
else
	if isnumeric(label)
		if isscalar(label) && label > 0
			label = sprintf('%i',label);
		else
			error(['The ''label'' argument must be either a character, ', ...
			'string, or a positive integer']);
		end
	elseif ~ischar(label)
		error(['The ''label'' argument must be either a character, ', ...
		'string, or a positive integer']);
	end
		
	% Find StratiGraph Window
	[b,nsg] = ismember(label, {MCS_PREF.sgwin(:).label});
	if b && WindowID.exist(java.lang.String(label))
		winID = MCS_PREF.sgwin(nsg).winID;
		if nargin == 1
      if isfield(MCS_PREF.sgwin(nsg),'com')
        sgc = MCS_PREF.sgwin(nsg).com;
      else
        % Create a new sgcom object and associate it to the window
        sgc = sgcom();
        MCS_PREF.sgwin(nsg).com = sgc;
      end
		else
			MCS_PREF.sgwin(nsg).com = sgc;
		end			
	else
		if b
			% The window has probably been closed, remove from list
			MCS_PREF.sgwin(nsg) = [];
		end
		error('No StratiGraph window with the label ''%s'' found.',label)
	end
end

end

%%-----------------------
function [b,k] = ismemberwin(id,ids)

b = false;
k = 0;
while ~b && k < length(ids)
  k = k+1;	
	if id == ids(k)
		b = true;
	end
end

end