function varargout = sgget(label)
%SGGET Get the active structure in StratiGraph.  
%   StructObj = SGGET returns a Matlab structure object of the active
%   structure in StratiGraph. If no structure or an edge is active,
%   StructObj is empty. 
%
%   ... = SGGET(Label) specifies the label of the StratiGraph window to 
%   get the data from. 
%
%   See also SGSET, SGGETSTART, SGOPEN.

narginchk(0,1);
nargoutchk(0,3);

if nargin == 0
	label = '';
end

[sgc, winID, label] = sgwindata(label);

% Check that MCS Toolbox Integrator is loaded into StratiGraph
import se.umu.stratigraph.core.PluginManager;
if ~PluginManager.isLoaded(winID,'MCS Toolbox Integrator')
	error('MCS:sgset:ExtensionNotLoaded',...
    'StratiGraph extension ''MCS Toolbox Integrator'' is not loaded for window ''%s''.',label)
end

import se.umu.stratigraph.Main;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataContainer;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataType;
import se.umu.stratigraph.plugin.extension.mcs.MCSRequest;
import se.umu.stratigraph.core.plugin.Result;
  
try 
	% Call se.umu.stratigraph.plugin.extension.mcs.MCSRequestDataContainer
	jrc  = MCSRequestDataContainer(MCSRequestDataType.GET_STRUCTURE);
	% Call se.umu.stratigraph.plugin.extension.mcs.MCSRequest
	jre  = MCSRequest(winID, jrc);
		
	jres = Main.callExtension(jre); % Get Result
	jstrCont = jres.getData(); % Get StructureContainer
	if ~jstrCont.hasData()
		str = [];
	else
		str = jstruct2mlstruct(jstrCont.getStructure(), jstrCont.getEigenvalues());
	end
catch e
	if isa(e,'matlab.exception.JavaException')
		ex = e.ExceptionObject; % Java exception
		if isa(ex,'java.lang.IllegalStateException')
			error('MCS:sgget:GetData','StratiGraph returned the error:\n  %s', char(ex.getMessage()))
		else
			rethrow(e)
		end
	else
		rethrow(e)
	end
end

varargout{1} = str;

if isempty(str)
  % If SG structure is empty clear sgcom strcuture
  sgc = sgcom();
  sgwindata(winID,sgc);
  [varargout{2:nargout}] = deal([]);
  warning('MCS:sgget:StructureCleared',...
    'No active canonical structure in the StratiGraph window %s. The associated canonical structure in Matlab is cleared.',label)
  return
elseif ~strcmp(class(str),class(getstruct(sgc)))
  % If SG structure and sgcom structure are not the same type,
  % assign th SG structure to the sgcom structure
  sgc = sgcom(str,jstrCont.getStrata().getSupClass());
  sgwindata(winID,sgc);
  [varargout{2:nargout}] = deal([]);
  warning('MCS:sgget:StructureChanged',...
    'The associated canonical structure in Matlab to the StratiGraph window %s has been changed to %s.',label,class(str))
  return
end
